Bulletin.prg

*  Program:   Bulletin.prg  (Bulletin Board Service)
*  Author:    David R. Alison
*  Version:   Clipper Summer '87
*  Notes:     Use this program as a central bulletin board
*             service on a network.  Demonstrates FREAD().
*
*             Libraries: CLIPPER.LIB, EXTEND.LIB
*
*  Public Domain software.

PUBLIC memo2read, frame
frame = 'Ŀ'     && Create the frame for the boxes.

*  Set up the main loop for the program.
DO WHILE 1 = 1
SET CURSOR OFF
i = 0

*  Set up the initial screen.
*  Notice the use of the ASCII CHRs 176 for shadow and 178 for
*  standard background.
FillScr()

*  Draw the title box and drop shadow.
@ 1,9 CLEAR TO 3,72
@ 1,9,3,72 BOX frame
@ 2,8 SAY CHR(176)
@ 3,8 SAY CHR(176)
@ 4,8 SAY REPLICATE( CHR(176), 63 )
@ 2,29 SAY 'Bulletin Board Service'

*  Load all bulletin files into an array.  If a file has the
*  extension BLT, it will be loaded into the array.
number = ADIR('*.blt')
IF number = 0          && Check to see if no bulletins are
   Errorbeep()         && available.
   CLEAR
   SET CURSOR ON
   ? "ERROR: No bulletins available.  Bulletins must be saved"
   ? "       as text files with the extension BLT."
   INKEY(0)
   CANCEL
ENDIF
DECLARE blt_files[number], blt_date[number], blt_name[number]
ADIR('*.blt', blt_files, '', blt_date )

*  Open each bulletin and read the first line into the array
*  blt_name.  This will be used for the menu selection of
*  which bulletin the user would like to read.
FOR i = 1 TO number
  block = 50
  buffer = space(50)
  filename = blt_files[i]
  handle = FOPEN(filename)
  IF FERROR() <> 0     && Make sure the file is readable.
     CLEAR
     SET CURSOR ON
     Errorbeep()
     ? 'ERROR: Cannot read file, DOS error ' + str(ferror())
  ELSE
     bytes = FREAD(handle, @buffer, block)
     IF bytes <> block
        CLEAR
        SET CURSOR ON
        Errorbeep()
        ? 'ERROR: Cannot read file ' + blt_files[i]
        INKEY(0)
        RETURN
     ELSE
        buffer = MEMOTRAN( buffer, CHR(250), CHR(250) )
        IF AT( CHR(250), buffer ) <> 0
           blt_name[i] = LEFT((TRIM(SUBSTR(buffer, 1, + ;
                         AT( CHR(250), buffer ) -1 )) + ;
                         REPLICATE( CHR(250),50) ), 50 )
        ELSE
           blt_name[i] = buffer
        ENDIF
        blt_name[i] = blt_name[i] + '  ' + DTOC(blt_date[i])
     ENDIF
  ENDIF
  FCLOSE(handle)        && Make sure we close the file.
NEXT

*  Display the ACHOICE menu with the available bulletins.
@ 6,9 CLEAR TO 19,72
@ 6,9,19,72 BOX frame
@ 8,9 SAY CHR(195) + REPLICATE( CHR(196), 62 ) + CHR(180)
@ 17,9 SAY CHR(195) + REPLICATE( CHR(196), 62 ) + CHR(180)
FOR i = 7 TO 19
   @ i, 8 SAY CHR(176)
NEXT
@ 20,8 SAY REPLICATE( CHR(176), 63 )

*  Add in the menu highlights.
@ 7,11 SAY "Bulletin's available"
@ 7,63 SAY 'Updated'
@ 18,12 SAY 'Highlight a bulletin and press ' + ;
            'Enter or press Esc to exit'

menuchoice = ACHOICE( 9, 11, 16, 70, blt_name )
fillscr()
IF menuchoice = 0       && If they hit Esc, exit the program.
   SET CURSOR ON
   CANCEL
ELSE
   Memo2Read = MEMOREAD(blt_files[menuchoice])
   MemoScr()
   MEMOEDIT(Memo2Read,1,4,17,77,.f.,"",120)
ENDIF
ENDDO

FUNCTION FillScr
*  Fill the screen with the background character CHR 176.
PRIVATE i, j
FOR j = 0 TO 24
   @ j,0 SAY REPLICATE(CHR(178),80)
NEXT
RETURN(0)

FUNCTION ErrorBeep
*  Sound an error-style tone on the speaker.
*  Syntax:  ERRORBEEP()
PRIVATE i
FOR i = 1 TO 2
   TONE(300,1)
   TONE(499,1)
NEXT
RETURN(0)

FUNCTION MemoScr
*  This function lays out the borders and frame for the
*  MEMOEDIT() call.
PRIVATE i
FOR i = 2 TO 17
   @ i,2 SAY CHR(176)
NEXT
@ 18,2 SAY REPLICATE( CHR(176), 74 )
@ 1,3 CLEAR TO 17,77
@ 20,3 CLEAR TO 22,77
@ 20,3,22,77 BOX frame
@ 21,2 SAY CHR(176)
@ 22,2 SAY CHR(176)
@ 23,2 SAY REPLICATE( CHR(176), 74 )
@ 21,12 SAY 'Commands:     '+chr(27)+chr(18)+chr(26) + ;
            '     Page Up     Page Down     Esc (Exit)'
RETURN(0)

*  EOP Bulletin.prg
