*  FUNCTION CHKDATE()
*
*  Written 10/25/88  Steve R. Busey, Copyright 1988 All Rights Reserved
*
*  This function accepts a STRING TYPE input containing the date
*    in the following forms: 
*         MM.DD.YY      MM-DD-YY     MM/DD/YY  and  MMDDYY      
*         M.D.YY        M-D-YY       M/D/YY
*         MM.D.YY       MM-D-YY      MM/D/YY
*         M.DD.YY       M-DD-YY      M/DD/YY 
*    and verifies the integrity of the date (including leap year),
*    reformats it if necessary to MM/DD/YY, stores the new format
*    to the calling variable, & redisplays it to the screen/GET.
*
*  This was designed to allow quick date entry from the numeric keypad
*    by entering dates like 9.2.88 without resorting to arrows or space
*    keys, and at the same time prevent accidental entering of invalid
*    dates by setting a lower limit on the year (ie- so 09/02/88 doesn't
*    get entered as 09/02/08).
*
*  Sample Call:
*
*      STORE SPACE(8) TO MDATE
*      @ 12, 12 SAY "Date: " GET MDATE VALID CHKDATE(MDATE)
*                         (Note: MDATE is a STRING variable)
*
FUNCTION CHKDATE

   PARAMETER chek                     && this is a STRING being passed !!!

   SET COLOR TO W+/B,W+/B,W                  && OR YOUR GETS DEFAULT COLORS
   STORE ROW() TO row1                       && save row and column of
   STORE COL() TO col1                       && window for proper updating
   IF chek = "  /  /  " .OR. EMPTY(chek)
      RETURN .T.                             && if empty, send it back OK
   ENDIF
   chek = STRTRAN(chek," ","")               && replace spaces with nothing
***
   IF AT("/",chek) = 0 .AND. AT("-",chek) = 0 .AND. AT(".",chek) = 0
      &&
      && if no delimiters
      &&
      IF LEN(TRIM(chek)) <> 6                && bad if not 6 chars long
         TONE(100,4)
         RETURN .F.
      ENDIF
      chek = STUFF(chek,3,0,"/")             && add slash delimiters
      chek = STUFF(chek,6,0,"/")                       
   ENDIF
***
   IF AT("/",chek) <> 0 .OR. AT("-",chek) <> 0 .OR. AT(".",chek) <> 0
      &&
      && if slash, dash, or dot delimiter
      &&
      IF LEN(TRIM(chek)) < 6                 && too short if less than 6 char
         TONE(100,4)
         RETURN .F.
      ENDIF
      chek = STRTRAN(chek,"-","/")           && replace dashes with slashes
      chek = STRTRAN(chek,".","/")           && replace dots with slashes
      lft = AT("/",chek)                     && get pos of left slash
      rt =  RAT("/",chek)                    && get pos of right slash
      mnt = SUBSTR(chek,1,(lft-1))           && extract month string
      dy = SUBSTR(chek,(lft+1),(rt-lft-1))   && extract day string
      yr = SUBSTR(chek,(rt+1),2)             && extract year string
      &&
      && check for bad month, day, or year out of range (87 thru 99, in this case)
      &&
      IF (VAL(mnt)<1 .OR. VAL(mnt)>12) .OR. (VAL(dy)<1 .OR. VAL(dy)>31) .OR. (VAL(yr)<87 .OR. VAL(yr)>99)    
         TONE(100,4)
         RETURN .F.
      ELSEIF VAL(dy)>30 .AND. (VAL(mnt)=4 .OR. VAL(mnt)=6 .OR. VAL(mnt)=9 .OR. VAL(mnt)=11) 
         TONE(100,4)                         && 30 days hath september...
         RETURN .F.
      ELSEIF VAL(mnt) = 2 .AND. ((VAL(dy) > 29) .OR. (VAL(dy)>28 .AND. !(VAL(yr)/4=INT(VAL(yr)/4)))) 
         TONE(100,4)                         && check february including leap year
         RETURN .F.
      ELSE                                   && OK, take 'er thru, son
         STORE "  /  /  " TO chek            && clear the variable
         IF lft = 2
            mnt = STUFF(mnt,1,0,"0")         && pad month to 2 chars
         ENDIF
         IF (rt-lft) = 2
            dy = STUFF(dy,1,0,"0")           && pad day to 2 chars
         ENDIF
         chek = STUFF(chek,1,2,mnt)          && insert month in var
         chek = STUFF(chek,4,2,dy)           && insert day in var
         chek = STUFF(chek,7,2,yr)           && insert year in var
         fldname = READVAR()                 && find calling variable name
         STORE TRIM(chek) TO &fldname        && move var to calling variable
         @ row1, (col1-8) SAY &fldname       && redisplay
         RETURN .T.                          && pass go, collect $200
      ENDIF 
   ELSE
      TONE(100,4) 
      RETURN .F.                             && else, bad date
   ENDIF
****
