/* display.c */

#include "nandef.h"
#include "extend.h"
#include "display.h"
#include "dispsub.h"
#include "keys.h"


/*                                                                      */
/* d_chattr() replace the color attribute with a new one starting at    */
/* location x, y and going for length len.                              */
/*                                                                      */


void d_chattr(int x, int y, int len, int attr)
{
    int i, pos;

    pos = (y * 160) + (x * 2) + 1;          /* calc the screen memory coord */

    for (i = 0; i < len; i++, pos += 2)     /* write the new attribute value */
        d_dispch(d_vseg, pos, (char) attr);
}


/*                                                                      */
/*  d_keyin() gets the next key typed and does any translation needed.  */
/*  Some keys are converted to a common name - like the up arrow is     */
/*  converted to the UP value which also is the Ctrl-E value.  This     */
/*  allows the Wordstar-like control keys to be used.  Only extended    */
/*  keys are translated - the values of the defines were chosen to      */
/*  match up with the non-extended key codes.                           */
/*                                                                      */

unsigned char d_keyin()
{
    unsigned char ch;

    ch = (unsigned char) d_getkey();        /* get the next key              */
    if ( ch == 0x00 )                       /* check to see if it's extended */
    {
        ch = (unsigned char) d_getkey();    /* if so, read the second part */
        switch ( ch )                       /*  and convert it             */
        {
            case 75   : ch = LFT;    break;
            case 77   : ch = RGT;    break;
            case 72   : ch = UP;     break;
            case 80   : ch = DN;     break;
            case 71   : ch = HOME;   break;
            case 79   : ch = ENND;   break;
            case 73   : ch = PGUP;   break;
            case 81   : ch = PGDN;   break;
            case 62   : ch = F4;     break;
            case 117  : ch = CENND;  break;
            case 119  : ch = CHOME;  break;
            case 118  : ch = CPGDN;  break;
            case 132  : ch = CPGUP;  break;
            case 116  : ch = CRGT;   break;
            case 115  : ch = CLFT;   break;
            case 82   : ch = INS;    break;
            case 83   : ch = DEL;    break;
            case 68   : ch = F0;     break;
            case 59   : ch = F1;     break;
            case 60   : ch = F2;     break;
            case 61   : ch = F3;     break;
            case 63   : ch = F5;     break;
            case 64   : ch = F6;     break;
            case 65   : ch = F7;     break;
            case 66   : ch = F8;     break;
            case 67   : ch = F9;     break;
            case 113  : ch = AF0;    break;
            case 104  : ch = AF1;    break;
            case 105  : ch = AF2;    break;
            case 106  : ch = AF3;    break;
            case 107  : ch = AF4;    break;
            case 108  : ch = AF5;    break;
            case 109  : ch = AF6;    break;
            case 110  : ch = AF7;    break;
            case 111  : ch = AF8;    break;
            case 112  : ch = AF9;    break;
            case 94   : ch = CF1;    break;
            case 95   : ch = CF2;    break;
            case 96   : ch = CF3;    break;
            case 97   : ch = CF4;    break;
            case 98   : ch = CF5;    break;
            case 99   : ch = CF6;    break;
            case 100  : ch = CF7;    break;
            case 101  : ch = CF8;    break;
            case 102  : ch = CF9;    break;
            case 103  : ch = CF0;    break;
            case 120  : ch = ALT1;   break;
            case 121  : ch = ALT2;   break;
            case 122  : ch = ALT3;   break;
            case 123  : ch = ALT4;   break;
            case 124  : ch = ALT5;   break;
            case 125  : ch = ALT6;   break;
            case 126  : ch = ALT7;   break;
            case 127  : ch = ALT8;   break;
            case 128  : ch = ALT9;   break;
            case 129  : ch = ALT0;   break;
            case 130  : ch = ADASH;  break;
            case 131  : ch = AEQL;   break;
            case 30   : ch = ALTA;   break;
            case 48   : ch = ALTB;   break;
            case 46   : ch = ALTC;   break;
            case 32   : ch = ALTD;   break;
            case 18   : ch = ALTE;   break;
            case 33   : ch = ALTF;   break;
            case 34   : ch = ALTG;   break;
            case 35   : ch = ALTH;   break;
            case 23   : ch = ALTI;   break;
            case 36   : ch = ALTJ;   break;
            case 37   : ch = ALTK;   break;
            case 38   : ch = ALTL;   break;
            case 50   : ch = ALTM;   break;
            case 49   : ch = ALTN;   break;
            case 24   : ch = ALTO;   break;
            case 25   : ch = ALTP;   break;
            case 16   : ch = ALTQ;   break;
            case 19   : ch = ALTR;   break;
            case 31   : ch = ALTS;   break;
            case 20   : ch = ALTT;   break;
            case 22   : ch = ALTU;   break;
            case 47   : ch = ALTV;   break;
            case 17   : ch = ALTW;   break;
            case 45   : ch = ALTX;   break;
            case 21   : ch = ALTY;   break;
            case 44   : ch = ALTZ;   break;
            default   : ch = 0;      break;
        }
    }

    return ( ch );
}



/*                                                                         */
/* function d_getblock() reads the text file and returns the a block.      */
/*  the variables offset and buffsize tell it where to start reading and   */
/*  how many bytes to try to read.  if the block read in would not fill    */
/*  the buffer then the offset is adjusted so that the start or end of     */
/*  of the file is positioned at the head or tail of the buffer.           */
/*                                                                         */
/* it returns the offset into the file of the first byte of the buffer.    */
/*                                                                         */

long d_getblock(int infile, long offset)
{
                /* set the file pointer to the proper offset   */
                /*  and if an error occured then check to see  */
                /*  if a positive offset was requested, if so  */
                /*  then set the pointer to the offset from    */
                /*  the end of the file, otherwise set it from */
                /*  the beginning of the file.                 */

    d_fileseek(infile, offset, SEEK_SET);

    /* read in the file and set the buffer bottom variable equal */
    /*  to the number of bytes actually read in.                 */

    d_buffbot = d_fileread(infile, (char far *) d_buffer, BUFFERSIZE);

    /* if a full buffer's worth was not read in, make it full. */

    if (( d_buffbot != BUFFERSIZE ) && ( d_fsize > BUFFERSIZE ))
    {
        if ( offset > 0 )
            d_fileseek(infile, (long) -BUFFERSIZE, SEEK_END);
        else
            d_fileseek(infile, (long) BUFFERSIZE, SEEK_SET);

        d_buffbot = d_fileread(infile, (char far *) d_buffer, BUFFERSIZE);
    }

    /* return the actual file position */

    return(d_filepos(infile) - d_buffbot);
}

/*                                                                   */
/* d_buff_align makes sure the buffer top and bottom variables point */
/* to actual complete lines of text.                                 */
/*                                                                   */

void d_buff_align()
{
    int i;

    d_bufftop = 0;
    d_buffbot = BUFFERSIZE;

    if ( d_buffoffset != 0L )       /* if the d_buffoffset is otherthan 0    */
    {
        i = d_bufftop;              /* start at the top of the file and scan */
                                    /* forward until a CR is reached.        */

        while (( d_buffer[i] != (char) CR ) && ( i < d_buffbot ))
            i++;

        d_bufftop = i + 2;          /* skip past the CR/LF to the first char  */
    }

                                    /* if the buffer offset is not a complete */
                                    /* buffer's length away from the file end */

    if ( d_buffoffset + ((long) d_buffbot) != d_fsize )
    {
                                   /* if the file position of the last byte   */
                                   /*  of the buffer would end up past the    */
                                   /*  end of the file, then the buffer does  */
                                   /*  contain a complete buffer full and the */
                                   /*  buffer end pointer needs to be set to  */
                                   /*  the last character of the file.        */

        if ( d_buffoffset + ((long) d_buffbot) > d_fsize )
            d_buffbot = (int) (d_fsize - d_buffoffset);

        i = d_buffbot;             /* point the end of the buffer to a valid  */
                                   /* complete text line.                     */

        while (( d_buffer[i] != (char) CR ) && ( i > d_bufftop ))
            i--;

        d_buffbot = i + 2;         /* skip past the CR/LF to the first char   */
    }
}


/*                                                                      */
/* d_win_align takes the value for d_wintop and then figures out where  */
/* d_winbot would be.  if d_winbot would extend past the end of the     */
/* buffer, then the top of the window is adjusted to ensure that a full */
/* screen of text will appear.  This simplifies the cursor routines.    */
/*                                                                      */


void d_win_align(int height)
{
    int i;

    d_winbot = d_wintop;    /* find out if there is enough text for */
    i        = 0;           /* full window.                         */

    while (( d_winbot < d_buffbot ) && ( i < height ))
    {
        if ( d_buffer[d_winbot] == (char) CR )
            i++;
        d_winbot++;
    }

    if ( i < height )           /* if there is not a full window,       */
    {
        while ( d_buffer[d_winbot] != (char) LF )    /* then retrofit d_winbot   */
            d_winbot--;                     /* to the end of a line      */

        d_wintop = d_winbot;
        i        = 0;                       /* and setup d_wintop       */

        while (( d_wintop > d_bufftop ) && ( i <= height ))
        {
            if ( d_buffer[d_wintop] == (char) LF )
                i++;
            d_wintop--;
        }

        if ( d_wintop != d_bufftop )
            d_wintop += 2;
    }
}


/* this routine displays the actual text in the window.  This is done   */
/* by taking each line and placing it in a string.  the screen line     */
/* is then taken from the appropriate group of characters in the string.*/
/* this allows a window to page left-right across the buffer without    */
/* having to use any complex algorithm to calc the needed chars.        */

void d_disp_update(int offset, int scol, int sline, int width, int height, int maxlin)
{
    int line, col, pos, i;
    char s[MAXLINE];

    d_refresh = NO;
    line      = 0;

    while ( line < height )
    {
        /* calculate the initial position, this save execution time because */
        /*  each column is considered as a offset from the line start       */

        pos = ((sline + line) * 160) + (scol * 2);

                                              /* copy string to temp buffer */

        for (i = 0; d_buffer[offset] != (char) CR && offset <= d_winbot; i++, offset++)
            s[i] = d_buffer[offset];

        for (; i < maxlin; i++)               /* fill out with spaces       */
            s[i] = ' ';


                              /* place the proper characters onto the screen */

        for (i = d_wincol - 1, col = 0; i < d_wincol + width - 1; i++, col++)
            d_dispch(d_vseg, pos + (col * 2), s[i]);

        line   += 1;
        offset += 2;
    }
}


/* move the cursor one line down, checking for window and buffer scroll */

void d_linedown(int eline, int height, int infile)
{
    int k;
    long i, j;

    if ( d_winrow < eline )     /* if cursor not at last line */
        d_winrow += 1;
    else                        /* otherwise adjust the window top variable */
    {
        d_refresh = YES;
        k         = d_winbot;

        while (( d_buffer[k] != (char) CR ) && ( k <= d_buffbot ))
            k++;

        k += 2;

        if ( k <= d_buffbot )
        {
            d_winbot = k;
            k        = d_wintop;

            while ( d_buffer[k] != (char) CR )
                k++;

            d_wintop = k + 2;
        }
        else
            if ( (((long) d_buffbot) + d_buffoffset) < d_fsize && d_fsize > BUFFERSIZE)
            {
                i = d_buffoffset + d_wintop;
                j = i;

                if ( j > d_fsize )
                    j = d_fsize - ((long) BUFFERSIZE);

                d_buffoffset = d_getblock(infile, j);

                if ( i < d_buffoffset )
                    d_wintop = 0;
                else
                    d_wintop = ((int) (i - d_buffoffset));

                d_buff_align();
                d_win_align(height);

                d_linedown(eline, height, infile);
            }
    }
}


/* move the cursor one line up, checking for window and buffer scroll */

void d_lineup(int sline, int height, int infile)
{
    int k;
    long i, j;

    if ( d_winrow > sline )
        d_winrow -= 1;
    else
    {
        d_refresh = YES;
        k         = d_wintop - 3;

        while (( d_buffer[k] != (char) CR ) && ( k > d_bufftop ))
            k--;

        if ( k >= d_bufftop )
        {
            if ( d_buffer[k] == (char) CR )
                k += 2;

            d_wintop = k;
            k        = d_winbot - 3;

            while ( d_buffer[k] != (char) CR )
                k--;

            d_winbot = k + 2;
        }
        else
            if ( ((long) d_bufftop) + d_buffoffset > 0 && d_fsize > BUFFERSIZE )
            {
                i = d_buffoffset + d_wintop;
                j = d_buffoffset - ((long) (BUFFERSIZE / 2));

                if ( j < 0 )
                    j = 0;

                d_buffoffset = d_getblock(infile, j);
                d_wintop     = ((int) (i - d_buffoffset));

                d_buff_align();
                d_win_align(height);

                d_lineup(sline, height, infile);
            }
    }
}


/* go to the top of the file */

void d_filetop(int sline, int height, int infile)
{
    if ( d_buffoffset != 0 )
    {
        d_buffoffset = d_getblock(infile, 0L);

        d_buff_align();
    }

    d_refresh = YES;
    d_wintop  = (int) d_buffoffset;
    d_winrow  = sline;
    d_wincol  = 1;

    d_win_align(height);
}


/* goto the bottom of the file */

void d_filebot(int eline, int height, int infile)
{
    if ( (((long) d_buffbot) + d_buffoffset) < d_fsize && d_fsize > BUFFERSIZE )
    {
        d_buffoffset = d_getblock(infile, d_fsize + 1);

        d_buff_align();
    }

    d_refresh = YES;
    d_wintop  = d_buffbot - 3;
    d_winrow  = eline;
    d_wincol  = 1;

    d_win_align(height);
}


CLIPPER dispfile()
{
    int i, j, done, hlight, norm;
    int sline, scol, eline, ecol;
    int height, width, mcount;
    int infile, colinc, maxlin, brows;
    char rval[2], fname[15];
    char mstr[25];
    unsigned char ch;

#ifdef DEBUG
    unsigned d_dseg;
    char     d_dmode;
#endif

    d_vseg = d_vconfig();           /* get the video buffer segment address */

#ifdef DEBUG
    if ( d_vseg = CGASEG )          /* debug video segment is opposite of */
    {
        d_dseg  = MDASEG;           /* the current segment.               */
        d_dmode = VCGA;
    }
    else
    {
        d_dseg  = CGASEG;
        d_dmode = VMDA;
    }
#endif

    strcpy(fname, _parc(1));        /* retrieve the filename */

    infile = d_fileopen(fname, READONLY);   /* open the file as read-only */

    if ( infile == 0 )                      /* check for file open error */
    {
        rval[0] = (char) 0;     /* build the return string char 0 tells      */
        rval[1] = '\0';         /* the calling routine that an error occured */
    }
    else                        /* if file opened okay, then get parameters */
    {
        sline  = _parni(2);
        scol   = _parni(3);
        eline  = _parni(4);
        ecol   = _parni(5);
        j      = _parni(6);
        norm   = _parni(7);
        hlight = _parni(8);

        strcpy(mstr, _parc(9));

        brows = _parl(10);

        if ( _parinfo(11) == 0 )
            colinc = 1;
        else
            colinc = _parni(11);

        if ( _parinfo(12) == 0 )
            maxlin = 132;
        else
            maxlin = _parni(12);

        if ( maxlin > MAXLINE )
            maxlin = MAXLINE;

        width        = ecol - scol + 1;
        height       = eline - sline + 1;
        mcount       = strlen(mstr);
        d_bufftop    = 0;
        d_buffbot    = BUFFERSIZE;
        d_buffoffset = 0;
        d_winrow     = sline;
        d_wincol     = 1;
        d_wintop     = 0;
        d_winbot     = 0;
        d_fsize      = d_fileseek(infile, 0L, SEEK_END) - 1;

        d_fileseek(infile, 0L, SEEK_SET);   /* get the first block */

        if ( d_fsize < ((long) d_buffbot) ) /* if block less than buffersize */
            d_buffbot = (int) d_fsize;      /*  then set buffer bottom       */

        d_buffoffset = d_getblock(infile, (long) d_bufftop);

        d_buff_align();
        d_win_align(height);

        for (i = 1; i < j; i++)                 /* point line pointer to */
            d_linedown(eline, height, infile);  /* line passed by caller */

        done      = NO;
        d_refresh = YES;

        for (i = sline; i <= eline; i++)
            d_chattr(scol, i, width, norm);

        do
        {
            if ( d_refresh == YES )
                d_disp_update(d_wintop, scol, sline, width, height, maxlin);

            d_chattr(scol, d_winrow, width, hlight);
            d_gotoxy(scol, d_winrow);

#ifdef DEBUG
            for (i = 0; i < 2000; i++)
            {
               d_dispch(d_dseg, i + i, d_buffer[i]);
               d_dispch(d_dseg, i + i + 1, 7);
            }

            d_dispch(d_dseg, (d_wintop * 2) + 1, 120);
            d_dispch(d_dseg, (d_winbot * 2) + 1, 120);
#endif

            ch = d_keyin();

            d_chattr(scol, d_winrow, width, norm);

            switch (ch)
            {
#ifdef DEBUG
                 case F4    : if ( d_dmode == VCGA )
                                  d_dmode = VMDA;
                              else
                                  d_dmode = VCGA;

                              d_vmode(d_dmode);
                              d_refresh = YES;
                              break;
#endif
                 case DN    : if ( brows == YES )
                                  d_winrow = eline;

                              d_linedown(eline, height, infile);
                              break;

                 case UP    : if ( brows == YES )
                                  d_winrow = sline;

                              d_lineup(sline, height, infile);
                              break;

                 case LFT   : d_wincol -= colinc;
                              d_refresh = YES;

                              if ( d_wincol < 1 )
                                  d_wincol = 1;

                              break;

                 case RGT   : d_wincol += colinc;
                              d_refresh = YES;

                              if ( d_wincol > (maxlin - width) )
                                  d_wincol = maxlin - width;

                              break;

                 case HOME  : d_wincol  = 1;
                              d_refresh = YES;
                              break;

                 case ENND  : d_wincol  = maxlin - width;
                              d_refresh = YES;
                              break;

                 case CLFT  : d_wincol -= 16;
                              d_refresh = YES;

                              if ( d_wincol < 1 )
                                  d_wincol = 1;

                              break;

                 case CRGT  : d_wincol += 16;
                              d_refresh = YES;

                              if ( d_wincol > (maxlin - width) )
                                  d_wincol = maxlin - width;

                              break;

                 case PGUP  : for (i = d_winrow; i > sline; i--)
                                  d_lineup(sline, height, infile);

                              for (i = 0; i < height; i++)
                                  d_lineup(sline, height, infile);

                              break;

                 case PGDN  : for (i = d_winrow; i > eline; i--)
                                  d_linedown(eline, height, infile);

                              for (i = 0; i < height; i++)
                                  d_linedown(eline, height, infile);

                              break;

                 case CPGUP : d_filetop(sline, height, infile);
                              break;

                 case CPGDN : d_filebot(eline, height, infile);
                              break;

                 case RET   : done = YES;
                              break;

                 case ESC   : done = YES;
                              break;

                 default    : for (i = 0; i <= mcount; i++)
                                  if ( mstr[i] == (char) ch )
                                      done = YES;
                              break;
            }
        } while ( done == NO );

        d_fileclose(infile);        /* close the file */

        rval[0] = (char) ch;
        rval[1] = '\0';
    }

#ifdef DEBUG
    d_vmode(VCGA);
#endif

    _retc(rval);                    /* return key value to Clipper */
}
