/* display.h */

/* defines and prototypes for display.c */

/* #define DEBUG */

#define BUFFERSIZE 4096     /* maximum size of the file buffer */
#define MAXLINE    255      /* maximum size of a line          */

#define OFF 0
#define ON  (!OFF)
#define NO  0
#define YES (!NO)
#define OK  0

#define FEOF ((char) 26)

#define SEEK_END 2      /* fileseek control values */
#define SEEK_CUR 1
#define SEEK_SET 0

#define READONLY  0     /* fileopen control values */
#define WRITEONLY 1
#define READWRITE 2

#define MDASEG 0x0b000  /* mono video memory segment */
#define CGASEG 0x0b800  /* color video memory segment */
#define VMDA   7
#define VCGA   3

#define MBLOCK 0x010d   /* mono block cursor   */
#define MNORM  0x0c0d   /* mono normal cursor  */
#define CBLOCK 0x0107   /* color block cursor  */
#define CNORM  0x0607   /* color normal cursor */

#define NOCHANGE 999    /* an invalid attribute value causes */
                        /*   no change to the attribute byte */

long d_buffoffset;            /* offset into buffer of current line      */
long d_fsize;                 /* file size in bytes                      */
int  d_bufftop, d_buffbot;    /* first and last character in buffer      */
int  d_wintop, d_winbot;      /* first and last character in window      */
int  d_winrow, d_wincol;      /* row and column of window highlight      */
char d_refresh;               /* YES means refresh screen                */

char d_buffer[BUFFERSIZE];    /* file buffer                             */

unsigned d_vseg;              /* video segment variable                  */


/* prototypes */


CLIPPER dispfile(void);

unsigned char d_keyin(void);
void d_chattr(int x, int y, int len, int attr);
long d_getblock(int handle, long offset);
void d_buff_align(void);
void d_win_align(int height);
void d_disp_update(int offset, int col, int line, int width, int height, int maxlin);
void d_linedown(int line, int height, int infile);
void d_lineup(int line, int height, int infile);
void d_filetop(int line, int height, int infile);
void d_filebot(int line, int height, int infile);

#ifdef DEBUG
    void d_vmode(char mode);
#endif

/* this is defined because I found out that CLIPPER has a version   */
/*   strcpy linked in normally - this just shut's up the LINT check */

extern char *strcpy(char *, char *);
