***  ERRORSYS.PRG
***  Property of Nantucket Corporation
***  Modifications by Greg Lief (last mod: 11/25/89)
***  CompuServe ID: 72460,1760
***  Voice:         503-588-1815
***  Data:          503-588-7572
***
***
***  Basically the same as the ALTERROR.PRG provided by Nantucket.
***  Primary difference is that for non-recoverable errors, I display
***  a pleasant box on the screen (as opposed to the terse one-line
***  message that usually appears at the top of the screen).  I also
***  reprint the error information, along with the current date and
***  time, to the file ERROR.TXT.  This file is appended to rather than
***  overwritten each time, so numerous error messages may be tracked.
***  You may wish to include information specific to your system (e.g.,
***  network user name or station number, etc).
***
***  If you have any good ideas for modifications to this, change it and
***  upload it again.  Maybe together we can come up with the "ultimate"
***  error-handler.  If only we lived in Utopia, where there were no errors!
***
***  Other notes:
***
***  1) PRINTER ERRORS: recoverable - user may redirect output to a file.
***
***  2) DIVIDE BY ZERO: recoverable - returns zero.
***
***  3) DOSERROR #4 (not enough file handles): a message to that effect
***     is written to ERROR.TXT.
***
***  4) NETWORK FILE OPEN ERRORS: If a network error (NETERR()) is detected,
***     and the chunk of code is "USE ...", I return False.  Your program
***     should contain code to handle network situations gracefully - there
***     is no need to crash the program because a file could not be opened
***     on a network.  That's the whole point of a multi-user system, right?
***
***  5) CORRUPTED INDEX FILES: there is not too much that can be done
***     about this problem.  However, if you are like me, you probably
***     have some sort of routine that rebuilds all of your index files
***     (mine is called RE_INDEX()).  With this in mind, I test for the
***     presence of this function by testing its TYPE() - if it is not
***     undefined, I call it to rebuild the indeces and return True.
***     Maybe this is treading on thin ice, but it seems to work for me.
***
***  6) DATABASE REQUIRED: Clipper has the following known bug: if you USE
***     a database that has an attached memo (.dbt) file and the .dbt file
***     is not found, the database will not be opened, yet you will NOT be
***     given any error message... UNTIL you attempt an operation such as
***     an INDEX or PACK.  Therefore, when this error is encountered, I add
***     a reminder to the programmer to check for missing .dbt files.  (Dunno
***     about you, but this is the ONLY reason I have ever gotten this error!)
***
***  7) EXPRESSION ERRORS: Clipper passes the code model, as well as up to
***     three relevant parameters, to EXPR_ERROR().  I have tweaked this
***     enough to print this model, along with each of the parameters.
***     Since these parameters could be of any type, I had to include a
***     small UDF entitled Parm2Str(), which simply converts them to a
***     character string for printing properly to ERROR.TXT.  Having all
***     the parameters in front of you makes it much easier to zap the bug.
***
***  8) CRASH-NO-MORE: If you wanted to badly enough, you could make every
***     erro recoverable.  However, this has obvious detrimental effects,
***     the first being that you would become very sloppy in your coding,
***     smug in the knowledge that your program would never crash.  I
***     highly discourage this way of thinking!
***

***  Printer Errors
*
FUNCTION print_error
PARAM name, line
PRIVATE key, file

SET DEVICE TO SCREEN
@ 0, 0
@ 0, 0 SAY "Proc " + M->name + " line " + LTRIM(STR(M->line)) +;
			", printer not ready"
@ 1, 0 SAY "Press <I>gnore, <R>etry, <B>REAK, F<ile>, <Q>uit..."
DO WHILE .T.
   key = UPPER(CHR(INKEY(0)))
   DO CASE
   CASE M->key == "Q"
           QUIT
   CASE M->key == "I"
           @ 0,0
           @ 1,0
           RETURN .F.
   CASE M->key == "R"
           @ 0,0
           @ 1,0
           RETURN .T.
   CASE M->key == "B"
           @ 0,0
           @ 1,0
           BREAK
   CASE M->key == "F"
           @ 0, 0
           @ 1,0
           ACCEPT "Filename - " TO file
           SET PRINTER TO (M->file)
           @ 0,0
           @ 1,0
           RETURN .T.
   ENDCASE
ENDDO
RETURN .F.


***  Expression Errors
*
function expr_error
parameters name, line, info, model, _1, _2, _3

** return 0 if zero divide error
IF M->info = "zero divide"
   RETURN 0
ELSE
   goofed("Proc " +M->name +" line " + ltrim(str(M->line)) + ", " + ;
          M->info + ", Code Model: " + M->model + ;
          IF(TYPE('M->_1') != 'U', parm2str('M->_1'), '') + ;
          IF(TYPE('M->_2') != 'U', parm2str('M->_2'), '') + ;
          IF(TYPE('M->_3') != 'U', parm2str('M->_3'), ''))
   RETURN .F.
ENDIF


***  Convert Parameters _1, _2, and _3 to character string
*
FUNCTION parm2str
PARAMETER parm
PRIVATE ret_val
ret_val = CHR(13) + CHR(10) + 'Parameter ' + SUBSTR(parm, 5, 1) + ': {'
DO CASE
   CASE TYPE(parm) = 'L'
      ret_val = ret_val + IF(&parm., '.T.', '.F.')
   CASE TYPE(parm) = 'D'
      ret_val = ret_val + DTOC(&parm.)
   CASE TYPE(parm) = 'N'
      ret_val = ret_val + LTRIM(STR(&parm.))
   OTHERWISE
      ret_val = ret_val + &parm.
ENDCASE
RETURN (ret_val + '}')


***  Undefined Symbol Errors
*
function undef_error
parameters name, line, info, model, _1
goofed("Proc " +M->name +" line " + ltrim(str(M->line)) + ", " + M->info + " " +;
       M->model + ' ' + M->_1)
return .f.


***  Miscellaneous Errors
*
function misc_error
parameters name, line, info, model
goofed("Proc " +M->name +" line " +ltrim(str(M->line)) + ", " + M->info + " " +;
       M->model)
return .f.

***  File Open Errors
*
function open_error
parameters name, line, info, model, _1
*** first check for network errors
IF NETERR() .AND. M->model = "USE"
   RETURN .F.
ELSE
   goofed("Proc " +M->name +" line " + ltrim(str(M->line)) + ", " + M->info + ;
          + " " + M->_1 +" (" +ltrim(str(DOSERROR())) +")" + ;
          IF(DOSERROR() = 4, CHR(13) + CHR(10) + "** ensure enough available "+;
          "file handles in CONFIG.SYS and via SET CLIPPER **", ""))
ENDIF
return .f.

***  Database Errors
*
function db_error
parameters name, line, info
private rebuildntx, x
rebuildntx = 'RE_INDEX()'   && store like this to avoid creating a symbol
*** test for corrupted index files and also for presence of RE_INDEX()
IF UPPER(M->info) = 'INDEX FILE CORRUPTED' .AND. AT("U", TYPE(rebuildntx)) = 0
   x = &rebuildntx.
   RETURN .T.
ELSE
   private buffer, handle
   goofed("Proc " +M->name +" line " +ltrim(str(M->line)) +", " +M->info + ;
          IF(UPPER(M->info) = 'DATABASE REQUIRED', CHR(13) + CHR(10) + ;
          "** make sure you are not missing any memo (.dbt) files! **", ""))
   return .f.
ENDIF

***  Display error message for user and write specifics to ERROR.TXT
*
FUNCTION goofed
PARAMETER msg
SET DEVICE TO SCREEN
SET COLOR TO +W/R
@ 10, 8, 12, 71  BOX 'Ŀ '
@ 11, 10 SAY 'A system error has occurred - please consult your programmer'
TONE(880,1)
TONE(440,1)
TONE(220,1)
TONE(110,1)
CLOSE ALL
IF ! FILE('error.txt')
   handle = FCREATE('error.txt')
ELSE
   handle = FOPEN('error.txt', 2)
   FSEEK(handle, 0, 2)   && move to end of file
endif
buffer = 'System Error: ' + msg + CHR(13) + CHR(10) + ;
         'Date: ' + DTOC(DATE()) + '   Time: ' + TIME() + CHR(13) + CHR(10) +;
         REPLICATE('-',80) + CHR(13) + CHR(10)
FWRITE(handle, buffer)
FCLOSE(handle)
inkey(0)
SET CURSOR ON
SET COLOR TO
CLEAR
QUIT
RETURN .F.

* eof: ERRORSYS.PRG
