; 
; FOXDEL Macro for Brief -- Deletes .FOX files automatically when .PRG
; files are written to disk.
; 
; Copyright (C) 1989, J.P. Software.  All Rights Reserved.
; 
; Author:  Tom Rawson, 6/14/89
; Modified:  Y. Alan Griver  1/9/90
;            Now checks for FoxPro FXP files as well
; 
; License Information:  These macro files and associated documentation may
; be distributed and used free of charge for any non-commercial purpose.
; They may not be used for any commercial purpose or distributed for a
; fee without the express written permission of J.P. Software.  This
; includes distribution by commercial disk vendors.  Contact J.P.
; Software at P.O. Box 1470, E. Arlington, MA 02174, USA, phone
; 617-646-3975, CompuServe 73577,243.
; 
; ==========================================================================
; 
; This documentation assumes you are familiar with Brief and at least
; to some extent with Brief macros.
; 
; The FOXDEL macro works by replacing Brief's write_buffer macro with a
; version that writes the buffer, then checks if the file being edited
; is a .PRG file.  If it is, the macro looks for a .FOX or .FXP file with
; the same name and in the same drive and directory.
; 
; If the .FOX or .FXP file is found, it is deleted, and the message "Deleted
; d:\path\filename.fox/fxp" is displayed in the message area.  If the .FOX
; or .FXP file exists but cannot be deleted for some reason (e.g. write-
; protected disk, read-only file, etc.) the message "Can't delete 
; d:\path\filename.fox" or "Can't delete d:\path\filename.fxp" is displayed
; instead.  If there is no .FOX or .FXP file, the standard "Write
; succesful" message will be displayed.
;
; To install FOXDEL succesfully you must make two changes to your
; system:
; 
;    (1) Put FOXDEL.CM wherever all your Brief macros are kept (usually
;    \BRIEF\MACROS).  You'll probably want to keep FOXDEL.M in the same
;    place.
; 
;    (2) At the end of your "initials" macro file, insert the code from
;    PRG.M (change the tab stops to your own liking) and then recompile
;    the initials macro.
; 
; Note that the new .prg macro gives an error message, "Can't find FOX
; file deletion macro", if FOXDEL.CM cannot be found when you start to
; edit a .PRG file.  If this happens you will be able to edit with no
; trouble, but the .FOX/.FXP files will not be deleted for you.  If it does
; happen it's probably because you didn't put FOXDEL.CM with your other
; .CM files or in a directory specified in your BPATH environment
; variable.
; 
; FoxBase Systems:
; FOXDEL is intended for systems where .FOX files are compiled
; periodically when the system reaches a stable point.  It provides a 
; safety valve to insure that the compiled copy of an an edited file
; has been deleted, so you are not running old code.  If you recompile
; .FOX files every time they are edited then the compiler will overwrite
; the old .FOX file with the new one; in that case FOXDEL is not
; necessary.


; FoxPro Systems:
; FoxPro now automatically creates the .FXP files when running a system.
; This fact makes this macro even more useful.  It provides a safety 
; valve to insure that the compiled copy of an edited file has been deleted,
; so that you are not running old code. 
; 

(macro foxdel                          ;dummy macro so we can tell foxdel
   (return)                            ;  is loaded
)

(macro write_buffer
   (
      (string full_name 
              extension
              fox_name)
      (if (<= (write_buffer) 0)
         (return)
      )
      (inq_names full_name extension)
      (if (== extension "prg")
         (
            (= fox_name (+ (substr full_name 1 (- (strlen full_name) 3)) "fox"))
            (if (exist fox_name)
               (if (> (del fox_name) 0)
                  (message (+ "Deleted " fox_name))
               ;else
                  (message (+ "Can't delete " fox_name))
               )
            )
            (= fox_name (+ (substr full_name 1 (- (strlen full_name) 3)) "fxp"))
            (if (exist fox_name)
               (if (> (del fox_name) 0)
                  (message (+ "Deleted " fox_name))
               ;else
                  (message (+ "Can't delete " fox_name))
               )
            )
         )
      )
   )
)
