#define TRUE   .T.
#define FALSE  .F.

/* ---------------------------------------------------------------------------

   Function:      MouseKey()

   Description:   Returns button press information with SET KEY TO support.

   Syntax:        <expN> = MouseKey( <expN> )

   Parameters:

    <expN>     -  The mouse button to be tested. MouseLib.ch contains the
                  #defines for the mouse buttons.

   Returns:

    <expN>     -  The InKey() value if any key pressed, otherwise zero

   Example:

   nKey = MouseKey( LBUTTON )

   ------------------------------------------------------------------------ */

FUNCTION MouseKey( nButton )

LOCAL nKey, bSetting

DO WHILE ( TRUE )

   nKey := INKEY()

   DO WHILE ( nKey = 0 )

      IF ( IsButton( nButton ) )
         EXIT
      ENDIF

      nKey := INKEY()

   ENDDO

   IF ( nKey != 0 )                    // If any key was pressed

      /*
          See if there is a set key to procedure. Execute it if there is
          one. Use procname(2), procline(2) so that the calling procedure
          name gets passed to the set key to routine
      */

      bSetting := SETKEY( nKey )

      IF( bSetting != NIL )
         EVAL( bSetting, procname(2), procline(2) )
      ELSE
         EXIT                          // otherwise return the key
      ENDIF

   ELSE
      EXIT
   ENDIF

ENDDO

RETURN ( nKey )

/* ---------------------------------------------------------------------------

   Function:      IsMouseKey()

   Description:   Returns whether a button or a key is pressed with
                  SET KEY TO support.

   Syntax:        IF IsMouseKey( <expN> )

   Parameters:

    <expN>     -  The mouse button to be tested. MouseLib.ch contains the
                  #defines for the mouse buttons.

   Returns:

    <expL>     -  Logical .T. if a mouse button or key pressed, otherwise
                  returns zero

   Example:

   IF IsMouseKey( LBUTTON )

   ------------------------------------------------------------------------ */
 
FUNCTION IsMouseKey( nButton )

LOCAL nKey, bSetting

DO WHILE ( TRUE )

   nKey := INKEY()

   DO WHILE ( nKey = 0 )

      IF ( IsButton( nButton ) )
         RETURN ( TRUE )
      ENDIF

      nKey := INKEY()

   ENDDO

   IF ( nKey != 0 )                    // If any key was pressed

      /*
          See if there is a set key to procedure. Execute it if there is
          one. Use procname(2), procline(2) so that the calling procedure
          name gets passed to the set key to routine
      */


      bSetting := SETKEY( nKey )

      IF( bSetting != NIL )
         EVAL( bSetting, procname(2), procline(2) )
      ELSE
         EXIT                          // otherwise return the key
      ENDIF

   ELSE
      EXIT
   ENDIF

ENDDO

RETURN ( FALSE )

// ----------------------------- EOF - MINKEY.PRG ----------------------------
