/****
*   Program Name : TBCELL.PRG                                                           
*
*   Author       : Michael Abadjiev   CIS 71563,3312
*   Last Update  : February 21, 1993                                                       
*
*   Purpose      : Tbrowse - everything fancy ypu can think about.
*                  You can browse any .DBF if you provide it as parameter,
*                  or the program will create a test file.
*   Language     : Clipper 5.01a
*
*   Compile      : clipper tbcell /n /w
*   Link         : rtlink fi tbcell
*
*   NOTE         : Many thanks to Mike Schinkel for his functions.
*
****/

/*
   Some extra NOTES:
   Clipper 5.01 exist more then 2 years I cann't  understand how some 
   programmers continue to use Rick Spence's preprocessed command DEFAULT
   I think that's the buggiest thing I have seen that's why I provide
   something more sophisticated.

   #xcommand DEFAULT <var> TO <val>        ;
   =>  if valtype(<var>) <> valtype(<val>) ;
       ; <var> := <val>                    ;
       ; endif

   Example: 

   default cClr to setcolor()
   default cText to "Clipper 5.01"
   default bBlk to {|| "Say what?"}
   default lResult to .f.

   etc.                                           

   I appreciate any suggestions especially for mouse routines in CLIPPER.
   Feel free to leave message to me.
   Thanks.
*/        

#include "inkey.ch"
#include "box.ch"
#include "setcurs.ch"
#include "getexit.ch"


#define KEY_ENTER   chr(17)+chr(196)+chr(217)       // The Enter sign
#define KEY_ARROWS  chr(26)+chr(24)+chr(25)+chr(27) // Up,Dn,Left,Right Arrows

// The Default Command
#xcommand DEFAULT <var> TO <val>        ;
=>  if valtype(<var>) <> valtype(<val>) ;
    ; <var> := <val>                    ;
    ; endif

// Centre text
#xtranslate CENTRETEXT( <row>, <msg>, <clr> ) 	      ;
=>     devpos(<row>,int((maxcol()+1-len(<msg>))/2)) ;;
       devout( <msg>, <clr> )

#xtranslate DATECONV( <date> )                        ;
=> alltrim( cdow( <date> ) ) +                        ;
           " " + cmonth(<date>) +                     ;
           " " + alltrim(str(day(<date>)) ) +         ;
           "," + str(year(<date>))

// Some sound
#xcommand BEEP ERROR    ;
=>  if set(_SET_BELL)   ; 
    ; tone(1200, .1)    ;
    ; tone(1400, .1)    ;
    ; tone(1600, .1)    ;
    ; endif

#xcommand BEEP ATTENTION    ;
=>  if set(_SET_BELL)       ;
    ; tone(1600, .1)        ;
    ; tone(1600, .1)        ;
    ; endif

#xcommand BEEP STOP        ;
=>  if set(_SET_BELL)      ;
    ; tone(100,.1)         ; 
    ; endif

// Get rid of warnings...
MEMVAR GetList


FUNCTION TestCell(cFile)

   FIELD text

   LOCAL nKey,                             ;
         i,                                ;
         b ,                               ;
         nCurs := setcursor(SC_NONE),      ;
         cBrowseColors,                    ;           
         cStab,                            ;
         cCol,                             ;
         cScreen,                          ;
         cTempScr,                         ;
         cOldScreen

   cOldScreen := savescreen(00,00,maxrow(),maxcol())
   set scoreboard off
   set bell on

   IF valtype(cFile) == "C" .and. !(at(".",cFile) == 0)
      IF !file(cFile)
         Beep Error
         alert("ERROR: "+ upper(cFile) + " Wrong file name!")
         setcolor("")
         scroll()
         return nil
      ENDIF
      cFile := substr( cFile, 1, at(".",cFile)-1 )
   ELSEIF valtype(cFile) == "C" 
      IF !file(cFile+".dbf")
         Beep Error
         alert("ERROR: File "+upper(cFile)+".DBF" + " doesn't exist!")
         setcolor("")
         cls
         quit 
      ENDIF
   ELSE
      // if the DBF is missing create the file....
      IF !file("TBCELL.DBF")
         CreateFile()
      END
      cFile := "TBCELL.DBF"
   END

   //                  1    2     3    4      5    6     7      8        9    10
   cBrowseColors := "W/B, B/W, W+/B, G+/B, BG+/B, GR+/R, R+/B, R+/BG, RB+/N, *GR+/N"

   b := tbrowsedb( 04,07,19,73 )
   b:colorSpec := cBrowseColors
   b:headsep := ""
   b:colsep  := "  "
   b:footsep := ""

   use (cFile) new                     
   
   // column #1 with the record #s......
   cCol := tbcolumnnew( padc("Record #",12), ;
   {|| padc(if(deleted(),"Del","    ") + str(recno(),5),12)} )
   cCol:colorBlock := { |v| ColumnColor(v) }
   b:addcolumn(cCol) 

                            
   // now build the TBrowse
   FOR i := 1  TO  fcount()
      IF valtype(eval(fieldblock( field(i) ))) == "M"
         cCol := tbcolumnnew(field(i),{|| "<Memo>" })
      ELSE
         cCol := tbcolumnnew( field(i),fieldblock( field(i)) )
      ENDIF
      cCol:colorBlock := { |v| ColumnColor(v) }
      b:addcolumn(cCol)                     
   NEXT i

   // freeze the first column 
   b:freeze := 1

   // screen fancy stuff...
   dispbegin()
   dispbox(00,00,maxrow()-1,maxcol(),replicate("",9),"N/W")
   @ 00,00 say padc("TBrowse Demo v2.0", maxcol()+1) ;
   color "N/BG"
   @ maxrow(),00 say padc( KEY_ARROWS + " Scroll, " + KEY_ENTER + ;
           " Edit, Del Toggle On/Off, Tab Refresh, Esc. Exit",80) color "N/BG"
   dispbox( 02,05,21,75,B_SINGLE+" ","BG+/B")
   Shadow( 02,05,21,75 )
   cScreen := savescreen(00,00,maxrow(),maxcol())
   cls
   dispend()

   // process keystrokes .....
   WHILE .t.
       IF cStab <> nil
          @ maxrow()-1,maxcol()-10 say "STABILIZING" color "N/W"
       ENDIF
       dispbegin()                                                  
       IF cScreen <> nil
          restscreen(00,00,maxrow(),maxcol(),cScreen)
          cScreen := nil
       ENDIF
       @ maxrow()-1,maxcol()-10 say replicate("",11) color "N/W"
       cStab := savescreen(maxrow()-1,maxcol()-10,maxrow()-1,maxcol())

       WHILE !b:stabilize()  ;   END                                                          
       dispend()
       restscreen(maxrow()-1,maxcol()-10,maxrow()-1,maxcol(),cStab)

       IF b:hittop .or. b:hitbottom  
          Beep Stop    
          clear typeahead
       END

       nKey := ClkInkey(0)
       DO CASE
       CASE nKey == K_ESC     
          // confirm before exit....
          Beep Attention
          IF alert("Do you really;want to QUIT?;"+replicate("",15),{"Yes","No"}) <> 2
             exit
          ENDIF
       CASE nKey == K_UP               ;   b:up()
       CASE nKey == K_DOWN             ;   b:down()
       CASE nKey == K_RIGHT            ;   b:right()
       CASE nKey == K_LEFT             ;   b:left()
       CASE nKey == K_ENTER            ;   CellEdit(b)
       CASE nKey == K_PGDN             ;   b:pagedown()
       CASE nKey == K_PGUP             ;   b:pageup()
       CASE nKey == K_CTRL_PGDN        ;   b:gobottom()
       CASE nKey == K_CTRL_PGUP        ;   b:gotop()
       CASE nKey == K_F10
           // save & draw the screen.....
           cTempScr := savescreen(12,30,15,50)
           dispbox(12,30,14,49,B_SINGLE+" ","G+/N")
           Shadow(12,30,14,49)
           CentreText(13,"Packing","G+/N")
           ClkInkey(.5)             
           // now we are ready to pack the file....
           pack  
           restscreen(12,30,15,50,cTempScr)
           // now refresh the TBrowse
           b:refreshall()

       CASE nKey == K_TAB
          // reconfigure, refreshall etc......
          cStab := savescreen(maxrow()-1,maxcol()-10,maxrow()-1,maxcol()+1)
          @ maxrow()-1,maxcol()-10 say "STABILIZING" color "N/W"
          dispbegin()   
          // at this point you can add/delete columns etc.....
          b:configure()
          b:refreshall()
          WHILE nextkey() == 0
             IF b:stabilize()
                exit
             ENDIF
          END                                                          
          dispend()
          restscreen(maxrow()-1,maxcol()-10,maxrow()-1,maxcol()+1,cStab)

       // play around with the deleted()....
       CASE nKey == K_DEL              
		  if(deleted(),dbrecall(),dbdelete())
         // refressh after the job is done....
         b:refreshcurrent()
       OTHERWISE
         Beep Stop
       ENDCASE
    END        

   setcolor("")
   setcursor(nCurs)                           
   dbcloseall()
   // credits etc.....
   Credits("Demo of Fancy Tbrowse v2.0")
   restscreen(00,00,maxrow(),maxcol(),cOldScreen)

RETURN nil


/****
*   Function CelEdit() -->LOGICAL
*
*   Author: Mike Schinkel - DBA 1992
****/

FUNCTION CellEdit(b)

   LOCAL c, uEditVar, nWidth, xScr
   LOCAL nRow := row(), nCol := col()
   LOCAL nTr, nLc, nBr, nRc
   LOCAL xConfirm := set(_SET_CONFIRM)
   LOCAL nCursor := setcursor()
   LOCAL lBlink := setblink()

   c := b:getcolumn(b:colpos)
   IF !TBColEditable(c) .or. deleted()
      Beep Error
   ELSE
      dispbegin()
      uEditVar := eval(c:block)
      nWidth := min(b:colwidth(b:colpos), DispLen(uEditVar))
      nTr := nRow - 1  ;   nLc := nCol -1
      nBr := nRow + 1  ;   nRc := nCol + nWidth
      xScr := savescreen( nTr, nLc, nBr+1, nRc+1 )
      dispbox( nTr, nLc, nBr, nRc,B_SINGLE,"BG+/B" )
      Shadow( nTr, nLc, nBr, nRc )
      setblink(.f.)
      @ nRow, nCol get uEditVar    ;
                   picture "@S" +  ;
                   alltrim(str(nWidth)) ;
                   color "B*/GR+"
      set(_SET_CONFIRM,.t.)    
      setcursor(SC_NORMAL)
      dispend()
      read                   
      dispbegin()
      setblink(lBlink)
      setcursor(nCursor)
      set(_SET_CONFIRM,xConfirm)
      IF lastkey() <> K_ESC
         eval( c:block,uEditVar )
         restscreen( nTr, nLc, nBr+1, nRc+1, xScr )
         b:refreshcurrent()
      ELSE
         restscreen( nTr, nLc, nBr+1, nRc+1, xScr )
      ENDIF
      dispend()
   ENDIF

RETURN nil

/****
*   Function TBColEdit() -->LOGICAL
*
*   Author: Mike Schinkel - DBA 1992
****/

STATIC FUNCTION TBColEditable(c)

   LOCAL uEditVar := eval(c:block), lResult := .f.

   eval( c:block, Modify(uEditVar) )
   IF !( uEditVar == eval(c:block) )
      eval( c:block, uEditVar )    
      lResult := .t.
   ENDIF

RETURN lResult

STATIC FUNCTION DispLen( uValue )

   LOCAL nLen := 0, cType := valtype( uValue )

   DO CASE
   CASE cType == "C"
      nLen := len( uValue )
   CASE cType == "D"
      nLen := len( set(_SET_DATEFORMAT ) )
   CASE cType == "L"
      nLen := 1
   CASE cType == "N"
      nLen := len( str( uValue ) )
   CASE cType == "U"
      nLen := 3
   ENDCASE

RETURN nLen


STATIC FUNCTION Modify( uValue )

   LOCAL cType := valtype( uValue )
   LOCAL uResult

   DO CASE
   CASE cType == "C"
      uResult := chr( asc( left( uValue,1 ) ) + 1 )
   CASE cType == "L"
      uResult := !uValue
   CASE cType == "D"
      uResult := uValue++
   CASE cType == "N"
      uResult := IF( uValue == 0, 1, 0 )
   ENDCASE

RETURN uResult



STATIC FUNCTION ColumnColor(uValue)

   LOCAL uType, cClr

     uType := valtype(uValue)

     DO CASE
     CASE uType == "N" .and. uValue < 0
       cClr := {7,8}
     CASE uType == "N" .and. uValue > 0
       cClr := {4,9}
     CASE uType = "L" .and. !uValue
       cClr := {7,9}
     CASE uType = "L" .and. uValue
       cClr := {4,5}
     CASE uType == "D"
       cClr := {5,6}
    CASE uType == "C" 
       IF !deleted()
       IF len(alltrim(uValue)) == 1
          cClr := {3,2}
       ELSEIF len(alltrim(uValue)) == 2
          cClr := {4,2}
       ELSEIF len(alltrim(uValue)) == 3
          cClr := {5,2}
       ELSE
          cClr := {1,2}
       END          
       ELSE
          cClr := {1,10}
       ENDIF
     OTHERWISE
       cClr := {1,2}
     endcase

RETURN cClr


FUNCTION Shadow( nT, nL, nB, nR ) 

    IF valtype(nT) <> "N" .or. valtype(nL) <> "N" .or. ;
       valtype(nB) <> "N" .or. valtype(nR) <> "N" 
       alert("ERROR: Wrong parameter(s)!;Function: Shadow()",{"Error"})
       RETURN nil
    ENDIF                               

    /*------------------------ line #: 1 -------------------------------*/
    restscreen( min(nB + 1, maxrow()), nL + 1, ;
                min(nB + 1, maxrow()), min(nR + 1, maxcol()),            ;
    transform( ;
    savescreen(min(nB + 1, maxrow()), ;
               nL + 1, min(nB + 1, maxrow()), min(nR + 1, maxcol())),    ;
    replicate("X" + chr(7), min(nR + 1, maxcol()) - nL + 2 ) ) )

    /*------------------------ line #: 2 -------------------------------*/
    restscreen( nT + 1, min(nR + 1, maxcol()), nB, min(nR + 1, maxcol()),;
    transform(                                                           ;
    savescreen(nT + 1, min(nR + 1, maxcol()), nB, min(nR + 1, maxcol())),;
    replicate("X" + chr(7), nB - nT + 2 ) ) )

RETURN nil


STATIC FUNCTION CreateFile()

   LOCAL aStructure := {     {"TEXT",    "C", 15, 0},       ;
                             {"DATES",   "D",  8, 0},       ;
                             {"LOGIC",   "L",  1, 0},       ;
                             {"NUMBERS", "N",  6, 2}        ;
                        },                                  ;
          aText :=      { "*",                 ;
                          "This is",                        ;
                          "an exaple ",                     ;
                          "how you can",                    ;
                          "have different",                 ;
                          "colors in the",                  ;
                          "TBrowse object"  },              ;
           aDel :=       { 1, 12, 45, 70, 100, 120, 150 }

   LOCAL i, j := 1, lLogical := .t.

   dispbegin()
   scroll()
   @ maxrow(),00 say padc("Creating: TBCELL.DBF",maxcol()+1) color  "N/W"
   @ maxrow(),maxcol() - 4  say "WAIT" color "N*/W"
   @ maxrow(),00 say "Record#:" color "N/W"
   dispend()

   // dbcreate() doesn't open the file!...
   dbcreate("TBCELL.DBF",aStructure)

   use tbcell new

   // Fill some junk just to show something
   FOR i:= 1 TO 150

      dbappend()
      tbcell->text := aText[j]
      j := IF(j == len(aText), 1, j+1 )
      tbcell->dates := date() + i
      tbcell->logic := !lLogical 
      lLogical := !lLogical
      tbcell->numbers := i - j
      dbcommit()
      @ maxrow(),10 say transform(recno(),"999") color "N/W"

   NEXT

   // Display few deleted record
   FOR i := 1 to len(aDel)
      tbcell->(dbgoto(aDel[i]))
      delete
   NEXT

   // Allways close before you go.....
   tbcell->(dbclosearea())

RETURN nil

FUNCTION Credits(cLine,lClear)

   LOCAL cScreen := savescreen(00,00,maxrow(),maxcol()),;
         nCursor := setcursor(SC_NONE),                 ;
         lBell   := set(_SET_BELL,.t.)

   default lClear to .f.
   default cLine to "Demo Fancy TBrowse"

   dispbegin()               
   IF lClear
   	dispbox(00,00,maxrow(),maxcol(),replicate(chr(176),9),"N/W")
   ENDIF
   dispbox(12,12,20,67,B_SINGLE+" ","W+/B")
   Shadow(12,12,20,67)
   CentreText(14,cLine,"BG+/B")
   CentreText(16,"Michael Abadjiev (c) 1993, Toronto, Canada","GR+/B")
   CentreText(17,"ph.(416) 431-0954","GR+/B")
   devpos(19,14) ; devout(version(),"BG+/B")
   CentreText(20,chr(16)+" Press any key "+chr(17),"W+/B")
   Beep Attention
   dispend()

   WHILE nextkey() == 0
      devpos(19,55) ; devout(ampm(time()),"GB+/B")
      ClockUpdate(0,maxcol()-12,"N/BG")
   END

   restscreen(00,00,maxrow(),maxcol(),cScreen)
   devpos(maxrow()-1,00)
   setcursor(nCursor)                 
   set(_SET_BELL,lBell)

RETURN nil 

/****
*   Function ClkInkey()-->inkey()
*
*   Purpose     : Inkey() replacement, handle also SET KEY TO (optional)
****/
    
FUNCTION ClkInkey( n,nRow,nCol,cClr )

    LOCAL nKey, bBlock

    default nRow to 0
    default nCol to maxcol()-12
    default cClr to "N/BG"

    IF pcount() == 0
          nKey = InKey()
    ELSEIF n == nil .or. n == 0    
        WHILE NextKey() == 0
            ClockUpdate(nRow,nCol,cClr)
        ENDDO    
        nKey := InKey()
     ELSE                 
        ClockUpdate(nRow,nCol,cClr)
        nKey := Inkey(n)
    ENDIF    
    
    IF (bBlock := setkey(nkey)) <> nil
         eval(bBlock)
         RETURN 0
    ENDIF

RETURN nKey 

/****
*   Function  ClockUpdate( .. ) --> nil 
*
*   Purpose     :  Update the clock
****/

FUNCTION ClockUpdate(nRow,nCol,cClr)

    LOCAL OldCursor := setcursor(0),;
          OldRow := row(), ;
          OldCol := col(), ;
          OldClr := setcolor()

   devpos(nRow,nCol)
   devout(ampm(time()),cClr)

   // reset all attributes
   devpos(OldRow,OldCol)
   setcolor(OldClr)
   setcursor(OldCursor)

RETURN nil 
    

/***
*	GetReader()
*	Standard modal read of a single GET.
*  Nantucket (c), some modifications Michael Abadjiev
*              
*  NOTE: In order to make the clock working we need to provide this function!
*/

proc GetReader( get )

local cClr := setcolor()

	// read the GET if the WHEN condition is satisfied
	if ( GetPreValidate(get) )

		// activate the GET for reading
		get:SetFocus()

		while ( get:exitState == GE_NOEXIT )

			// check for initial typeout (no editable positions)
			if ( get:typeOut )
				get:exitState := GE_ENTER
			end

			// apply keystrokes until exit
           while ( get:exitState == GE_NOEXIT )
               GetApplyKey( get, ClkInkey(0) )                        
			end

			// disallow exit if the VALID condition is not satisfied
			if ( !GetPostValidate(get) )
				get:exitState := GE_NOEXIT
			end

		end

		// de-activate the GET
		get:KillFocus()

	end

return

// **** EOF: tbcell.prg
